==> component-spring-2.1.0-sources.jar: com/bxm/newidea/component/rule/RuleGroupExecutor.java <==
package com.bxm.newidea.component.rule;

import com.bxm.newidea.component.annotations.RuleBean;
import com.bxm.newidea.component.exception.ExcutorException;
import com.bxm.newidea.component.tools.SpringContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.OrderComparator;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 规则分组执行器
 *
 * @author liujia
 * @date 1/13/21 6:03 PM
 **/
@Component
@Slf4j
public class RuleGroupExecutor implements ApplicationRunner {

    private Map<String, List<IRule>> groupRuleMap = new HashMap<>();

    private ThreadLocal<List<Class<? extends IRule>>> choiceRuleThreadLocal = new ThreadLocal<>();

    private ThreadLocal<List<Class<? extends IRule>>> skipRuleThreadLocal = new ThreadLocal<>();

    /**
     * 选择只执行某些过滤器，不在指定的过滤器列表中，则跳过
     *
     * @param filterClass 需要跳过的过滤器
     * @return 过滤器执行器
     */
    public RuleGroupExecutor choice(Class<? extends IRule> filterClass) {
        return setThreadLocal(filterClass, choiceRuleThreadLocal);
    }

    /**
     * 从执行的过滤器中跳过某些过滤器不执行
     *
     * @param filterClass 需要跳过的过滤器
     * @return 过滤器执行器
     */
    public RuleGroupExecutor skip(Class<? extends IRule> filterClass) {
        return setThreadLocal(filterClass, skipRuleThreadLocal);
    }

    private RuleGroupExecutor setThreadLocal(Class<? extends IRule> filterClass,
                                             ThreadLocal<List<Class<? extends IRule>>> threadLocal) {
        List<Class<? extends IRule>> ruleList = threadLocal.get();
        if (null == ruleList) {
            ruleList = new ArrayList<>();
        }
        ruleList.add(filterClass);

        threadLocal.set(ruleList);
        return this;
    }

    /**
     * 执行对应业务分组的规则
     *
     * @param groupName 业务分组名称，与{@link IRule}实现类上的注解保持一致
     * @param context   规则执行的上下文
     * @return 规则匹配结果
     */
    @SuppressWarnings("unchecked")
    public <T> boolean apply(String groupName, T context) {
        List<IRule> rules = groupRuleMap.get(groupName);

        if (null == rules) {
            log.error("不存在[{}]对应的逻辑规则分组", groupName);
            return false;
        }


        List<Class<? extends IRule>> choiceRuleList = this.choiceRuleThreadLocal.get();
        List<Class<? extends IRule>> skipRuleList = skipRuleThreadLocal.get();

        try {
            for (IRule rule : rules) {
                if (null != skipRuleList && skipRuleList.contains(rule.getClass())) {
                    continue;
                }

                if (null != choiceRuleList && choiceRuleList.size() > 0) {
                    if (!choiceRuleList.contains(rule.getClass())) {
                        continue;
                    }
                }

                if (!rule.apply(context)) {
                    return false;
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new ExcutorException("业务逻辑规则执行失败，规则分组：" + groupName, e);
        } finally {
            choiceRuleThreadLocal.remove();
            skipRuleThreadLocal.remove();
        }

        return true;
    }

    @Override
    public void run(ApplicationArguments args) {
        for (IRule rule : SpringContextHolder.getBeans(IRule.class)) {
            RuleBean annotation = AnnotationUtils.findAnnotation(rule.getClass(), RuleBean.class);
            if (null == annotation) {
                log.warn("[{}]未提供RuleBean注解", rule.getClass().getSimpleName());
                continue;
            }

            List<IRule> rules = groupRuleMap.get(annotation.group());
            if (rules == null) {
                rules = new ArrayList<>();
            }
            rules.add(rule);
            groupRuleMap.put(annotation.group(), rules);
        }

        for (List<IRule> rules : groupRuleMap.values()) {
            OrderComparator.sort(rules);
        }
    }
}

==> component-spring-2.1.18-sources.jar: com/bxm/newidea/component/rule/RuleGroupExecutor.java <==
package com.bxm.newidea.component.rule;

import com.bxm.newidea.component.annotations.RuleBean;
import com.bxm.newidea.component.exception.ExcutorException;
import com.bxm.newidea.component.tools.SpringContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.OrderComparator;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 规则分组执行器
 * 适用场景：
 * 需要进行连续判断业务前置条件时，可以将不同的判断逻辑单独定义为IRule的实现类，实现解耦和复用
 *
 * @author liujia
 * @date 1/13/21 6:03 PM
 **/
@Component
@Slf4j
public class RuleGroupExecutor implements ApplicationRunner {

    private Map<String, List<IRule>> groupRuleMap = new HashMap<>();

    private ThreadLocal<List<Class<? extends IRule>>> choiceRuleThreadLocal = new ThreadLocal<>();

    private ThreadLocal<List<Class<? extends IRule>>> skipRuleThreadLocal = new ThreadLocal<>();

    /**
     * 选择只执行某些规则，不在指定的规则列表中，则跳过
     *
     * @param filterClass 需要跳过的规则
     * @return 规则执行器
     */
    public RuleGroupExecutor choice(Class<? extends IRule> filterClass) {
        return setThreadLocal(filterClass, choiceRuleThreadLocal);
    }

    /**
     * 从执行的规则中跳过某些规则不执行
     * 优先级会高于{@link #choice(Class)}方法
     *
     * @param filterClass 需要跳过的业务规则
     * @return 规则执行器
     */
    public RuleGroupExecutor skip(Class<? extends IRule> filterClass) {
        return setThreadLocal(filterClass, skipRuleThreadLocal);
    }

    private RuleGroupExecutor setThreadLocal(Class<? extends IRule> filterClass,
                                             ThreadLocal<List<Class<? extends IRule>>> threadLocal) {
        List<Class<? extends IRule>> ruleList = threadLocal.get();
        if (null == ruleList) {
            ruleList = new ArrayList<>();
        }
        ruleList.add(filterClass);

        threadLocal.set(ruleList);
        return this;
    }

    /**
     * 执行对应业务分组的规则
     *
     * @param groupName 业务分组名称，与{@link IRule}实现类上的注解保持一致
     * @param context   规则执行的上下文
     * @return 规则匹配结果，true表示全部规则符合
     */
    @SuppressWarnings("unchecked")
    public <T> boolean apply(String groupName, T context) {
        List<IRule> rules = groupRuleMap.get(groupName);

        if (null == rules) {
            log.error("不存在[{}]对应的逻辑规则分组", groupName);
            return false;
        }


        List<Class<? extends IRule>> choiceRuleList = this.choiceRuleThreadLocal.get();
        List<Class<? extends IRule>> skipRuleList = skipRuleThreadLocal.get();

        try {
            for (IRule rule : rules) {
                if (null != skipRuleList && skipRuleList.contains(rule.getClass())) {
                    continue;
                }

                if (null != choiceRuleList && choiceRuleList.size() > 0) {
                    if (!choiceRuleList.contains(rule.getClass())) {
                        continue;
                    }
                }

                if (!rule.apply(context)) {
                    return false;
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new ExcutorException("业务逻辑规则执行失败，规则分组：" + groupName, e);
        } finally {
            choiceRuleThreadLocal.remove();
            skipRuleThreadLocal.remove();
        }

        return true;
    }

    @Override
    public void run(ApplicationArguments args) {
        for (IRule rule : SpringContextHolder.getBeans(IRule.class)) {
            RuleBean annotation = AnnotationUtils.findAnnotation(rule.getClass(), RuleBean.class);
            if (null == annotation) {
                log.warn("[{}]未提供RuleBean注解", rule.getClass().getSimpleName());
                continue;
            }

            List<IRule> rules = groupRuleMap.get(annotation.group());
            if (rules == null) {
                rules = new ArrayList<>();
            }
            rules.add(rule);
            groupRuleMap.put(annotation.group(), rules);
        }

        for (List<IRule> rules : groupRuleMap.values()) {
            OrderComparator.sort(rules);
        }
    }
}
